/**	
	$Author: dizzy $
	$Revision: 1.4 $
	Last updated: $Date: 2000/09/13 11:50:27 $

	Copyright (C) 2000 Vlad Mereuta <V.C.Mereuta@warwick.ac.uk>

	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 2 of the License, or
	any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA	
	
	$Log: AccountDetailsForm.c,v $
	Revision 1.4  2000/09/13 11:50:27  dizzy
	The transaction module is fully functional
	
	Revision 1.3  2000/09/04 14:23:07  dizzy
	General code cleanup
	
	Revision 1.1  2000/08/23 12:08:29  dizzy
	Added details dialog for accounts

*/

#include "AccountDetailsForm.h"

/// the selected category
static UInt16	selected_cat=dmUnfiledCategory;
/// selected category name
static Char		cat_name [dmCategoryLength];
/// this form
static FormPtr	frm;


Boolean AccDetails_Save()
{	
	accountStructure		 	a;
	packedAccountStructure* 	pas;
	MemHandle					rech;
	Int32						newbal;
	UInt32						uid;

	rech=DmGetRecord(AccountsDB_ref,AccountsDB_current_record);
	ErrFatalDisplayIf(!rech,"rech null");
	pas=(packedAccountStructure*) MemHandleLock(rech);
	AccountsDB_UnpackAccount(&a,pas);
	MemHandleUnlock(rech);

	//if the balance is invalid it will be 0
	newbal=String2Pence((Char*) GetTextPtr(ACC_DETAILS_BALANCE));
	a.credit_limit=String2Pence((Char*) GetTextPtr(ACC_DETAILS_CREDIT));
	a.owner=(Char*) GetTextPtr(ACC_DETAILS_OWNER);
	if(!a.owner) return false;
	a.bank=(Char*) GetTextPtr(ACC_DETAILS_BANK);
	if(!a.bank) return false;
	
	a.acname=(Char*) GetTextPtr(ACC_DETAILS_NAME);
	if(!a.acname) return false;

	//write the data
	AccountsDB_PackAccount(&a,rech);

	AccountsDB_SetRecordCategory(AccountsDB_current_record,selected_cat);
	//release record and set the dirty bit
	DmReleaseRecord(AccountsDB_ref, AccountsDB_current_record, true); 

	if (newbal!=a.balance) { 
		DmRecordInfo(AccountsDB_ref,AccountsDB_current_record,NULL,&uid,NULL);
		AccountsDB_UpdateBalance(uid,newbal);
	}
	return true;
}

void AccDetails_Init()
{
	ControlPtr					ctl;
	MemHandle					rech;
	packedAccountStructure* 	pas;
	accountStructure 			as;
	UInt16						attr;
	Char*						s;
	

	rech=DmQueryRecord(AccountsDB_ref,AccountsDB_current_record);
	pas=(packedAccountStructure*) MemHandleLock(rech);

	if(!pas) return;

	AccountsDB_UnpackAccount(&as,pas);

	// init the category
	DmRecordInfo(AccountsDB_ref,AccountsDB_current_record,&attr,NULL,NULL);
	attr&=dmRecAttrCategoryMask;	
	selected_cat=attr;
	
	ctl = GetObjectPtrByForm (frm,ACC_DETAILS_TYPE_TRIGGER);
	CategoryGetName (AccountsDB_ref,selected_cat, cat_name);
	CategorySetTriggerLabel(ctl, cat_name);

	SetFieldTextFromStr(ACC_DETAILS_OWNER,as.owner);
	SetFieldTextFromStr(ACC_DETAILS_BANK,as.bank);
	SetFieldTextFromStr(ACC_DETAILS_NAME,as.acname);

	s=Pence2String(as.balance);
	SetFieldTextFromStr(ACC_DETAILS_BALANCE,s);
	s=Pence2String(as.credit_limit);
	SetFieldTextFromStr(ACC_DETAILS_CREDIT,s);

	MemHandleUnlock(rech);

	//free memory allocated by Pence2String
	MemPtrFree(s);
}

void	AccDetails_SelectCat()
{
	UInt16		category;
	Boolean 	categoryEdited;
	
	// Process the category popup list.  
	category = selected_cat;

	categoryEdited=CategorySelect(AccountsDB_ref,frm, ACC_DETAILS_TYPE_TRIGGER,ACC_DETAILS_TYPE_LIST,\
		false,&category,cat_name,1,categoryDefaultEditCategoryString);
	selected_cat=category;
//	ErrFatalDisplay("selectcat");
}

Boolean AccDetails_HandleEvent(EventPtr event)
{
	Boolean 	handled = true;

	// Get the form pointer
	frm = FrmGetActiveForm();

	// Key Events
	switch(event->eType) {

	// Form Open
	case frmOpenEvent:		
		AccDetails_Init();

		//draw form
		FrmDrawForm(frm);
		break;
	
	// Controls
	case ctlSelectEvent:
		switch( event->data.ctlSelect.controlID ) {
			case ACC_DETAILS_TYPE_TRIGGER:		//Select a category
				AccDetails_SelectCat();
				handled = true;
				break;
			case ACC_DETAILS_OK:
				handled = true;
				//if something went wrong do nothing
				if(! AccDetails_Save()) break;
				//update the main form
				FrmUpdateForm(ACCOUNTS_MAIN_FORM,0);				
				FrmReturnToForm(ACCOUNTS_MAIN_FORM);
				break;
			
			case ACC_DETAILS_CANCEL:
				//cancel button was pressed;
				//simply return to the main form
				handled = true;
				FrmUpdateForm(ACCOUNTS_MAIN_FORM,0);				
				FrmReturnToForm(ACCOUNTS_MAIN_FORM);
				break;

			case ACC_DETAILS_DELETE:
				//delete button
				DmRemoveRecord(AccountsDB_ref,AccountsDB_current_record);
				handled = true;
				FrmUpdateForm(ACCOUNTS_MAIN_FORM,0);				
				FrmReturnToForm(ACCOUNTS_MAIN_FORM);
				break;
				
			default:
				handled = false;
		};
		break;	
	// Everything else...
	default:
		handled = false;
	}	
	return handled;
}
