/**	
	$Author: dizzy $
	$Revision: 1.7 $
	Last updated: $Date: 2000/09/13 11:50:27 $
	
	$Log: AccountsDB.c,v $
	Revision 1.7  2000/09/13 11:50:27  dizzy	

	Copyright (C) 2000 Vlad Mereuta <V.C.Mereuta@warwick.ac.uk>

	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 2 of the License, or
	any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

//define this to ignore the external definition of the vars below in the header file
#define __ACCOUNTS_DB_C__

#include "TransactionsDB.h"


//the variables below have to be accessed in different object files

DmOpenRef	AccountsDB_ref;
UInt32		AccountsDB_num_cat;
UInt16		AccountsDB_current_cat=dmAllCategories;
Int16		AccountsDB_current_record;
totals 		AccountsDB_totals={true,0,0};

void	AccountsDB_UpdateTotals()
{
	UInt16					recordNo,i;
	MemHandle				rech;
	packedAccountStructure*	pas;
	accountStructure 		as;
		
	if(AccountsDB_totals.dirty) {
		//we have to update the totals
		//scan through the database adding the balances from each account
		
		AccountsDB_totals.dirty=false;
		AccountsDB_totals.total=AccountsDB_totals.eom_total=0;
		
	  	for(i=0,recordNo=0;i<AccountsTable_total_items;i++,recordNo++) {
			//fetch record
			rech=DmQueryNextInCategory(AccountsDB_ref,&recordNo,AccountsDB_current_cat);		
			ErrFatalDisplayIf(!rech, "rech is 0 - aborting");
	
			pas=(packedAccountStructure*) MemHandleLock(rech);
			ErrFatalDisplayIf(!pas, "pas is 0 - aborting");
			if(!pas) return;

			AccountsDB_UnpackAccount(&as,pas);
		
			AccountsDB_totals.total+=as.balance;
			AccountsDB_totals.eom_total+=as.eom_balance;
				
			MemHandleUnlock(rech);
		}
	}
}

MemHandle AccountsDB_GetUnpackedRecord(UInt32 id,accountStructure* as)
{
	Int16						record;
	MemHandle					rech;
	packedAccountStructure* 	pas;

	DmFindRecordByID(AccountsDB_ref,id,&record);
	
	rech=DmQueryRecord(AccountsDB_ref,record);
	if(!rech) return NULL;
	ErrFatalDisplayIf(!rech, "rech is 0 - aborting");

	pas=(packedAccountStructure*) MemHandleLock(rech);
	ErrFatalDisplayIf(!pas, "pas is 0 - aborting");
	if(!pas) return NULL;

	AccountsDB_UnpackAccount(as,pas);
	MemHandleUnlock(rech);
	return rech;
}


Int32	AccountsDB_GetBalance(UInt32 id)
{
	Int16						record;
	MemHandle					rech;
	packedAccountStructure* 	pas;
	Int32						ret_val;

	DmFindRecordByID(AccountsDB_ref,id,&record);

	rech=DmQueryRecord(AccountsDB_ref,record);
	ErrFatalDisplayIf(!rech, "rech is 0 - aborting");

	pas=(packedAccountStructure*) MemHandleLock(rech);
	ErrFatalDisplayIf(!pas, "pas is 0 - aborting");

	ret_val=pas->balance;
	MemHandleUnlock(rech);
	return ret_val;
}

Int32	AccountsDB_GetEomBalance(UInt32 id)
{
	Int16						record;
	MemHandle					rech;
	packedAccountStructure* 	pas;
	Int32						ret_val;

	DmFindRecordByID(AccountsDB_ref,id,&record);

	rech=DmQueryRecord(AccountsDB_ref,record);
	ErrFatalDisplayIf(!rech, "rech is 0 - aborting");

	pas=(packedAccountStructure*) MemHandleLock(rech);
	ErrFatalDisplayIf(!pas, "pas is 0 - aborting");

	ret_val=pas->eom_balance;
	MemHandleUnlock(rech);
	return ret_val;
}

void AccountsDB_SetRecordCategory(UInt16 index, UInt16 cat)
{
	db_SetRecordCategory(AccountsDB_ref,index,cat);
}

void AccountsDB_UnpackAccount (accountStructure* account, const packedAccountStructure* packedAccount)
{
	Char*	s = packedAccount->owner;
	
	account->balance = packedAccount->balance;
	account->eom_balance = packedAccount->eom_balance;
	account->credit_limit = packedAccount->credit_limit;

	account->owner = s;
	s += StrLen(s) + 1;
	account->bank = s;
	s += StrLen(s)+1;
	account->acname = s;
}

void AccountsDB_PackAccount (accountStructure* account, MemHandle accountDBEntry)
{
	// figure out necessary size
	UInt32	length = 0;
	MemPtr	s;
	UInt32	offset = 0;

	length = sizeof(account->balance) + sizeof(account->eom_balance) + sizeof(account->credit_limit)+\
		 StrLen(account->owner) + StrLen(account->bank) + StrLen(account->acname) + 3;  // 4 for 2 string terminators

	// resize the VoidHand
	if (MemHandleResize(accountDBEntry, length) == 0) {
		// copy the fields
		s = MemHandleLock(accountDBEntry);
		offset = 0;
		DmWrite(s, offset, (MemPtr) &account->balance,sizeof(account->balance));
		offset += sizeof(account->balance);
		DmWrite(s, offset, (MemPtr) &account->eom_balance,sizeof(account->eom_balance));
		offset += sizeof(account->eom_balance);
		DmWrite(s, offset, (MemPtr) &account->credit_limit,sizeof(account->credit_limit));
		offset += sizeof(account->credit_limit);
		
		DmStrCopy(s, offset, (Char*) account->owner);
		offset += StrLen(account->owner) + 1;
		DmStrCopy(s, offset, (Char*) account->bank);
		offset += StrLen(account->bank)+1;
		DmStrCopy(s, offset, (Char*) account->acname);
		
		MemHandleUnlock(accountDBEntry);
	}
}

Err		AccountsDB_Init()
{
	UInt32	mode = dmModeReadWrite;
	Err	err = 0;
	MemPtr	c;
	Boolean	created;

	palmCoinsAppInfoType* 	appInfoP;

	// Find the database.  If it doesn't exist, create it.
	db_OpenOrCreate(&AccountsDB_ref, ACCOUNTSDB_TYPE,PALM_COINS_CREATOR, mode,0, ACCOUNTSDB_NAME, &created);
	//if the db was created put in some sample values
	//have to initialise the application info
	c = AccountsDB_GetLockedAppInfo();
	if (created) AccountsDB_InitializeAccounts();

	MemPtrUnlock(c);
	appInfoP = (palmCoinsAppInfoType*) c;
	
	return err;
}

void	AccountsDB_Close()
{
	DmCloseDatabase(AccountsDB_ref);
}


MemPtr	AccountsDB_GetLockedAppInfo()
{
	UInt16	cardNo;
	LocalID	dbID;
	LocalID	appInfoID;
	Err	err;

	err = DmOpenDatabaseInfo(AccountsDB_ref, &dbID, NULL, NULL,&cardNo, NULL);
	ErrFatalDisplayIf (err,  "Get getting application info block");
	err = DmDatabaseInfo(cardNo, dbID, NULL, NULL, NULL, NULL, NULL,NULL, NULL, &appInfoID, NULL, NULL, NULL);
	ErrFatalDisplayIf (err,  "Get getting application info block");
	
	if(appInfoID==NULL) return CoinsAppInfoInit(dbID,cardNo);
	//a bit useless - to be removed unless code will be inserted here
	return MemLocalIDToLockedPtr(appInfoID, cardNo);
}

void	AccountsDB_InitializeAccounts()
{
	UInt16 i;
	
	accountStructure a1 = {20000,15000,0,"JDe","Natwest","Test acc"};
	accountStructure a4 = {2200,1500,0,"John Doe","Galactic Bank-West","Test1 acc"};

	accountStructure a2 = {0,0,0,"n/a","n/a","E:Misc"};
	accountStructure a3 = {0,0,0,"n/a","n/a","I:Misc"};
	accountStructure a5 = {0,0,0,"n/a","n/a","I:Balance update"};
	accountStructure a6 = {0,0,0,"n/a","n/a","E:Balance update"};

	accountStructure a7 = {0,0,0,"n/a","n/a","I:Interest"};	
	accountStructure a8 = {0,0,0,"n/a","n/a","I:Salary"};
	accountStructure a9 = {0,0,0,"n/a","n/a","I:Gift"};
	accountStructure a10 = {0,0,0,"n/a","n/a","I:Alimony"};
	
	accountStructure a51 = {0,0,0,"n/a","n/a","E:Utilities"};
	accountStructure a52 = {0,0,0,"n/a","n/a","E:U:Gas"};
	accountStructure a53 = {0,0,0,"n/a","n/a","E:U:TV"};
	accountStructure a54 = {0,0,0,"n/a","n/a","E:U:Phone"};
	accountStructure a55 = {0,0,0,"n/a","n/a","E:U:Eletricity"};
	accountStructure a56 = {0,0,0,"n/a","n/a","E:U:Rent"};

	accountStructure a57 = {0,0,0,"n/a","n/a","E:Travel"};
	accountStructure a58 = {0,0,0,"n/a","n/a","E:T:Land"};
	accountStructure a59 = {0,0,0,"n/a","n/a","E:T:Sea"};
	accountStructure a60 = {0,0,0,"n/a","n/a","E:T:Air"};
	accountStructure a61 = {0,0,0,"n/a","n/a","E:T:Meals"};

	accountStructure a62 = {0,0,0,"n/a","n/a","E:Food"};
	accountStructure a63 = {0,0,0,"n/a","n/a","E:F:Dining"};
	accountStructure a64 = {0,0,0,"n/a","n/a","E:Clothing"};
	accountStructure a65 = {0,0,0,"n/a","n/a","E:Gifts"};
	accountStructure a66 = {0,0,0,"n/a","n/a","E:Computer"};
	accountStructure a67 = {0,0,0,"n/a","n/a","E:Books"};
	accountStructure a68 = {0,0,0,"n/a","n/a","E:Entretainment"};
		
	
	
	//do this to force setting the category to income before the 1st transaction is to be recorded
	AccountsDB_current_cat=CategoryFind(AccountsDB_ref,"Income");
	i=AccountsDB_NewRecord(&a3);
	AccountsDB_current_cat=dmAllCategories;
	
	//default test accounts
	i=AccountsDB_NewRecord(&a1);
	AccountsDB_SetRecordCategory(i,CategoryFind(AccountsDB_ref,"Bank"));
	i=AccountsDB_NewRecord(&a4);
	AccountsDB_SetRecordCategory(i,CategoryFind(AccountsDB_ref,"Bank"));
	
	//misc and balance update - used by the program
	i=AccountsDB_NewRecord(&a2);
	AccountsDB_SetRecordCategory(i,CategoryFind(AccountsDB_ref,"Expense"));
	i=AccountsDB_NewRecord(&a5);
	AccountsDB_SetRecordCategory(i,CategoryFind(AccountsDB_ref,"Income"));
	i=AccountsDB_NewRecord(&a6);
	AccountsDB_SetRecordCategory(i,CategoryFind(AccountsDB_ref,"Expense"));

	//other income accounts - modify as you see fit
	i=AccountsDB_NewRecord(&a7);
	AccountsDB_SetRecordCategory(i,CategoryFind(AccountsDB_ref,"Income"));
	i=AccountsDB_NewRecord(&a8);
	AccountsDB_SetRecordCategory(i,CategoryFind(AccountsDB_ref,"Income"));
	i=AccountsDB_NewRecord(&a9);
	AccountsDB_SetRecordCategory(i,CategoryFind(AccountsDB_ref,"Income"));
	i=AccountsDB_NewRecord(&a10);
	AccountsDB_SetRecordCategory(i,CategoryFind(AccountsDB_ref,"Income"));

	//other expense accounts
	i=AccountsDB_NewRecord(&a51);
	AccountsDB_SetRecordCategory(i,CategoryFind(AccountsDB_ref,"Expense"));
	i=AccountsDB_NewRecord(&a52);
	AccountsDB_SetRecordCategory(i,CategoryFind(AccountsDB_ref,"Expense"));
	i=AccountsDB_NewRecord(&a53);
	AccountsDB_SetRecordCategory(i,CategoryFind(AccountsDB_ref,"Expense"));
	i=AccountsDB_NewRecord(&a54);
	AccountsDB_SetRecordCategory(i,CategoryFind(AccountsDB_ref,"Expense"));
	i=AccountsDB_NewRecord(&a55);
	AccountsDB_SetRecordCategory(i,CategoryFind(AccountsDB_ref,"Expense"));
	i=AccountsDB_NewRecord(&a56);
	AccountsDB_SetRecordCategory(i,CategoryFind(AccountsDB_ref,"Expense"));
	i=AccountsDB_NewRecord(&a57);
	AccountsDB_SetRecordCategory(i,CategoryFind(AccountsDB_ref,"Expense"));
	i=AccountsDB_NewRecord(&a58);
	AccountsDB_SetRecordCategory(i,CategoryFind(AccountsDB_ref,"Expense"));
	i=AccountsDB_NewRecord(&a59);
	AccountsDB_SetRecordCategory(i,CategoryFind(AccountsDB_ref,"Expense"));
	i=AccountsDB_NewRecord(&a60);
	AccountsDB_SetRecordCategory(i,CategoryFind(AccountsDB_ref,"Expense"));
	i=AccountsDB_NewRecord(&a61);
	AccountsDB_SetRecordCategory(i,CategoryFind(AccountsDB_ref,"Expense"));
	i=AccountsDB_NewRecord(&a62);
	AccountsDB_SetRecordCategory(i,CategoryFind(AccountsDB_ref,"Expense"));
	i=AccountsDB_NewRecord(&a63);
	AccountsDB_SetRecordCategory(i,CategoryFind(AccountsDB_ref,"Expense"));
	i=AccountsDB_NewRecord(&a64);
	AccountsDB_SetRecordCategory(i,CategoryFind(AccountsDB_ref,"Expense"));
	i=AccountsDB_NewRecord(&a65);
	AccountsDB_SetRecordCategory(i,CategoryFind(AccountsDB_ref,"Expense"));
	i=AccountsDB_NewRecord(&a66);
	AccountsDB_SetRecordCategory(i,CategoryFind(AccountsDB_ref,"Expense"));
	i=AccountsDB_NewRecord(&a67);
	AccountsDB_SetRecordCategory(i,CategoryFind(AccountsDB_ref,"Expense"));
	i=AccountsDB_NewRecord(&a68);
	AccountsDB_SetRecordCategory(i,CategoryFind(AccountsDB_ref,"Expense"));
			
}

void AccountsDB_UpdateBalance(UInt32 uid,Int32 newbal)
{
	accountStructure			a;
	transactionStructure		t;

	AccountsDB_GetUnpackedRecord(uid,&a);
	
	if (a.balance < newbal) { //adjust by adding money
		//record the opening account transaction
		t.source_acc=AccountsDB_FindAccountByName("I:Balance update",CategoryFind(AccountsDB_ref,"Income"));
		ErrFatalDisplayIf(!t.source_acc,"not found");
		t.dest_acc=uid;
		t.ammount=newbal-a.balance;
	}
	else { //adjust by taking away money
		t.dest_acc=AccountsDB_FindAccountByName("E:Balance update",CategoryFind(AccountsDB_ref,"Expense"));
		ErrFatalDisplayIf(!t.dest_acc,"not found");
		t.source_acc=uid;
		t.ammount=a.balance-newbal;
	}
		
	t.payee="Balance update";
	DateSecondsToDate(TimGetSeconds(),& t.date);
	t.cleared=true;
	t.method=pm_transfer;
	t.note="";	
	TransactionsDB_NewRecord(&t);	
}	


UInt32	AccountsDB_FindAccountByName(char* n, UInt16 categ)
{
	UInt16				recordNo;
	UInt32				uniqueID;
	accountStructure	as;

  	for(recordNo=0;;recordNo++) {
		if(!DmQueryNextInCategory(AccountsDB_ref,&recordNo,categ)) return 0;		
		DmRecordInfo(AccountsDB_ref,recordNo,NULL,&uniqueID,NULL);

		AccountsDB_GetUnpackedRecord(uniqueID,&as);

		if(!StrCaselessCompare(n,as.acname)) return uniqueID;
	}
	
}

UInt16	AccountsDB_NewRecord(accountStructure* s)
{
	MemHandle				h;
	UInt16					index=0;
	transactionStructure	ts;
	Int32					bal;
	
	//clear the balance since it is maintained via the transaction
	bal=s->balance;
	s->balance=s->eom_balance=0;
	
	h=DmNewRecord(AccountsDB_ref, &index,1);
	if (h) 	AccountsDB_PackAccount(s,h);
		else 	ErrFatalDisplay("No space left?");
	DmReleaseRecord(AccountsDB_ref, index, true); //true sets the dirty bit	
	
	if(AccountsDB_current_cat!=dmAllCategories) AccountsDB_SetRecordCategory(index,AccountsDB_current_cat);

	//record the opening account transaction
	ts.source_acc=AccountsDB_FindAccountByName("I:Misc",CategoryFind(AccountsDB_ref,"Income"));
	ErrFatalDisplayIf(!ts.source_acc,"not found");
	ts.payee="Account opened";
	//save uniques id of the new record
	DmRecordInfo(AccountsDB_ref,index,NULL,&ts.dest_acc,NULL);
	DateSecondsToDate(TimGetSeconds(),& ts.date);
	ts.cleared=true;
	ts.ammount=bal;
	ts.method=pm_transfer;
	ts.note="";	
	TransactionsDB_NewRecord(&ts);
	
	return index;
}

MemPtr CoinsAppInfoInit(LocalID dbID,UInt16 cardNo)
{
	UInt16					wordValue;
	LocalID					appInfoID;
	palmCoinsAppInfoType*	nilP = 0;
	palmCoinsAppInfoType* 	appInfoP;

	MemHandle h;

	
	h = DmNewHandle(AccountsDB_ref, sizeof (palmCoinsAppInfoType));
	ErrFatalDisplayIf (!h,  "Cannot allocate app info block");

	//get the local id of the newly allocated appinfo record
	appInfoID = MemHandleToLocalID (h);
	//record the id
	DmSetDatabaseInfo(cardNo, dbID, NULL, NULL, NULL, NULL, NULL, NULL,NULL, &appInfoID, NULL, NULL, NULL);
	
	appInfoP = MemLocalIDToLockedPtr(appInfoID, cardNo);

	// Clear the app info block.
	DmSet (appInfoP, 0, sizeof(palmCoinsAppInfoType), 0);

	// Initialize the categories.
	CategoryInitialize ((AppInfoPtr) appInfoP, PALM_COINS_CATEGORIES);

	// I don't know what this field is used for.
	wordValue = 0xFFFF;
	DmWrite (appInfoP, (UInt32)&nilP->dirtyAppInfo, &wordValue,
		sizeof(appInfoP->dirtyAppInfo));

	// Initialize the sort order.
	DmSet (appInfoP, (UInt32)&nilP->sortOrder, sizeof(appInfoP->sortOrder), 
		soName);

	return appInfoP;
}
