/**	
	$Author: dizzy $
	$Revision: 1.7 $
	Last updated: $Date: 2000/09/13 11:50:27 $
	
	$Log: AccountsTable.c,v $
	Revision 1.7  2000/09/13 11:50:27  dizzy
	The transaction module is fully functional
	
	Revision 1.6  2000/09/08 10:41:10  dizzy
	Transactiondatabas initial support and column display preferences support
	
	Revision 1.5  2000/09/04 14:23:07  dizzy
	General code cleanup

	Copyright (C) 2000 Vlad Mereuta <V.C.Mereuta@warwick.ac.uk>

	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 2 of the License, or
	any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
	
*/

#define __ACCOUNTS_TABLE_C__

#include "AccountsTable.h"

// the uniqueid for top visible items
UInt16		AccountsTable_top_visible_item=0;
// the max no of rows that can be displayed
UInt16		AccountsTable_rows;
// no of elements that could be loaded in the table
UInt16		AccountsTable_total_items;

//name all the columns
UInt16		AccountsTable_name_col=0;
UInt16		AccountsTable_eom_balance_col=1;
UInt16		AccountsTable_balance_col=2;

// the table
static TablePtr		table;


void AccountsTable_ShowScrollArrows()
{
    ControlPtr	ptrup, ptrdown;
    Boolean		enable;
	char 		*pageupchr="\001", *pagedownchr="\002";
	char 		*pageupgreychr="\003", *pagedowngreychr="\004";

    ptrup = GetObjectPtr(ACC_PGUP);
    ptrdown = GetObjectPtr(ACC_PGDN);
    if ((AccountsTable_total_items > AccountsTable_rows) || AccountsTable_top_visible_item) {
    	// first element is not 0, hence we can scroll up
        if (AccountsTable_top_visible_item) {
            CtlSetLabel(ptrup, pageupchr);
            enable = true;
        }
        else {
            CtlSetLabel(ptrup, pageupgreychr);
            enable = false;
        }
        
		CtlShowControl(ptrup);
        CtlSetEnabled(ptrup, enable);
	
		// enable scrolling down if there are enough elements
        if ((AccountsTable_top_visible_item+AccountsTable_rows) < AccountsTable_total_items) {
            CtlSetLabel(ptrdown, pagedownchr);
            enable = true;
        }
        else {
            CtlSetLabel(ptrdown, pagedowngreychr);
            enable = false;
        }
        CtlShowControl(ptrdown);
        CtlSetEnabled(ptrdown, enable);
    }
    else {
		CtlSetEnabled(ptrdown,false);
		CtlSetEnabled(ptrup,false);
        CtlHideControl(ptrup);
        CtlHideControl(ptrdown);
    }
}

void AccountsTable_DrawTableNumber(void* table, Int16 row, Int16 column, RectangleType* bounds)
{
	Char	*s;

	if(column==AccountsTable_eom_balance_col) s=Pence2String(AccountsDB_GetEomBalance(TblGetRowData(table,row)));
		else s=Pence2String(AccountsDB_GetBalance(TblGetRowData(table,row)));

	WinDrawChars(s,StrLen(s),bounds->topLeft.x,bounds->topLeft.y);

	//free memory allocated by Pence2String
	MemPtrFree(s);
}

void AccountsTable_DrawTableText(void* table, Int16 row, Int16 column, RectangleType* bounds)
{
	accountStructure 			as;

	AccountsDB_GetUnpackedRecord(TblGetRowData(table,row),&as);
	
	WinDrawChars(as.acname,StrLen(as.acname),bounds->topLeft.x,bounds->topLeft.y);
}

void AccountsTable_LoadData()
{
	UInt16		recordNo=AccountsTable_top_visible_item,row;
	UInt32		uniqueID;

	TblUnhighlightSelection(table);
	
	AccountsTable_total_items=DmNumRecordsInCategory(AccountsDB_ref,AccountsDB_current_cat);
	//show the scrolling arrows if necessary
	AccountsTable_ShowScrollArrows(AccountsTable_total_items);
	
  	for(row=0;row<AccountsTable_rows;row++) {
		//if we run out of items to display just set the rows to unusable
		if(row+AccountsTable_top_visible_item>=AccountsTable_total_items) {
			TblSetRowUsable(table,row,false);
			continue;
		} else TblSetRowUsable(table,row,true);
		
		//fetch next record
		DmQueryNextInCategory(AccountsDB_ref,&recordNo,AccountsDB_current_cat);		
		
		DmRecordInfo(AccountsDB_ref,recordNo,NULL,&uniqueID,NULL);
		TblSetRowData(table,row,uniqueID);
		
		//start searching from the next loop the following time
		recordNo++; 
	}
}

void AccountsTable_Unhilight()
{
	TblUnhighlightSelection(table);

}

void AccountsTable_Init()
{
	UInt16		row;
	
	//init globals
	table=(TablePtr) GetObjectPtr(ACC_TABLE);
	
	AccountsTable_Unhilight();
	
	AccountsTable_total_items=DmNumRecordsInCategory(AccountsDB_ref,AccountsDB_current_cat);

	ErrFatalDisplayIf(!table, "table is null!!!");

	//the numbers are drawn with our custom procedure
	TblSetCustomDrawProcedure(table, AccountsTable_name_col, AccountsTable_DrawTableText);
	TblSetCustomDrawProcedure(table, AccountsTable_eom_balance_col, AccountsTable_DrawTableNumber);
	TblSetCustomDrawProcedure(table, AccountsTable_balance_col, AccountsTable_DrawTableNumber);

	//get max number of visible rows
	AccountsTable_rows = TblGetNumberOfRows(table);

	//set style for all cells
	for (row = 0; row < AccountsTable_rows; row++) {
		TblSetItemStyle(table, row, AccountsTable_name_col, customTableItem);
		TblSetItemStyle(table, row, AccountsTable_eom_balance_col, customTableItem);
		TblSetItemStyle(table, row, AccountsTable_balance_col, customTableItem);
	}
	
	//set all columns as usable
	TblSetColumnUsable(table, AccountsTable_name_col, true);
	if(coins_pref.disp_eom_col) {
		TblSetColumnUsable(table, AccountsTable_eom_balance_col,true);
		TblSetColumnWidth(table,AccountsTable_name_col,66);
	}
	else {
		//dont display the column
		TblSetColumnUsable(table, AccountsTable_eom_balance_col,false);
		//make the anme column bigger
		TblSetColumnWidth(table,AccountsTable_name_col,111);
	}
		
	TblSetColumnUsable(table, AccountsTable_balance_col, true);
	
	AccountsTable_LoadData();
}
