/**	
	$Author: dizzy $
	$Revision: 1.7 $
	Last updated: $Date: 2000/09/13 11:50:27 $
	
	$Log: MiscFunc.c,v $
	Revision 1.7  2000/09/13 11:50:27  dizzy
	The transaction module is fully functional
	
	Revision 1.6  2000/09/08 10:39:52  dizzy
	Databases are now opened by name.
	
	Revision 1.5  2000/09/04 14:23:07  dizzy
	General code cleanup
	
	Revision 1.3  2000/08/23 12:08:30  dizzy
	Added details dialog for accounts
	
	Revision 1.2  2000/08/21 15:04:20  dizzy
	Fixed more bugs. Added scrolling support
	
	Revision 1.1  2000/08/21 08:59:30  dizzy
	Reorganised files

	Copyright (C) 2000 Vlad Mereuta <V.C.Mereuta@warwick.ac.uk>

	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 2 of the License, or
	any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA	
*/

#include "MiscFunc.h"

Err	db_OpenOrCreate(DmOpenRef *dbP, UInt32 type, UInt32 creator, UInt16 mode, UInt16 cardNo, Char* name, Boolean* created)
{
	Err	err=0;

	*created = false;
	
	if (! DmFindDatabase(cardNo,name) ) { //database does not exist
		err = DmCreateDatabase(0, name, creator, type, false);
		if (err) { 
			return err;
			ErrFatalDisplayIf(err,"create db failed db");
		}
		*created = true; //set flag
	}

	//open database (has to exist at this point
	*dbP = DmOpenDatabase(cardNo,DmFindDatabase(cardNo,name),mode);
	if (! *dbP) return DmGetLastErr();

	return err;
}

void db_SetRecordCategory(DmOpenRef db, UInt16 index, UInt16 cat)
{
	UInt16	attr;

	DmRecordInfo(db,index,&attr,NULL,NULL);
	attr &= ~dmRecAttrCategoryMask;	
	attr|=cat;
	DmSetRecordInfo(db,index,&attr,NULL);			
}

UInt16 db_GetRecordCategory(DmOpenRef db, UInt16 index)
{
	UInt16	attr;

	DmRecordInfo(db,index,&attr,NULL,NULL);
	attr &= dmRecAttrCategoryMask;	
	return attr;
}

Char*	Pence2String(Int32 pence)
{
	Char* s=MemPtrNew(30);
	Char* t=MemPtrNew(30);

	StrIToA(s,pence/100);
	StrCat(s,".");
	pence%=100; if(pence<0) pence*=-1;
	StrIToA(t,pence);
	if(pence<10) StrCat(s,"0");
	StrCat(s,t);
	MemPtrFree(t);
	return s;
}

Int32	String2Pence(Char* sum)
{
	Int32	res,p;
	Char*	pence=StrChr(sum,'.');

	//returns 0 if the pointer is 0
	if(!sum) return 0;
	
	if(pence) {
		//split the pence part from the main sum part
		pence[0]=0; //overwrite the dot
		pence++;
	}
	p=StrAToI(pence);
	
	
	res=StrAToI(sum);
	
	if(p<10 && StrLen(pence)!=2) p*=10;

	res*=100;
	if(res<0) res-=p;
		else res+=p;
	return res;
}

void*	GetObjectPtr(UInt16 objID)
{
	FormPtr frm;
	frm=FrmGetActiveForm();
	return (FrmGetObjectPtr (frm, FrmGetObjectIndex (frm,objID)));
}

void*	GetObjectPtrByForm(FormPtr frm,UInt16 objID)
{
	return (FrmGetObjectPtr (frm, FrmGetObjectIndex (frm,objID)));
}

void*	GetTextPtr(UInt16 objID)
{
	FieldPtr fp=(FieldPtr) GetObjectPtr(objID);
	
	return FldGetTextPtr(fp);
}

FieldPtr SetFieldTextFromHandle(Int16 fieldID, MemHandle txtH)
{
   MemHandle	oldTxtH;
   FormPtr	frm = FrmGetActiveForm();
   FieldPtr	fldP;


   // get the field and the field's current text handle.
   fldP = GetObjectPtrByForm(frm, fieldID);
   ErrNonFatalDisplayIf(!fldP, "missing field");
   oldTxtH = FldGetTextHandle(fldP);
   
   // set the field's text to the new text.
   FldSetTextHandle(fldP, txtH);
   FldDrawField(fldP);

   // free the handle AFTER we call FldSetTextHandle().
   if (oldTxtH) 
      MemHandleFree(oldTxtH);
   
   return fldP;
} 

FieldPtr SetFieldTextFromStr(Int16 fieldID, const Char* str)
{
   MemHandle      txtH;
   
   // get some space in which to stash the string.
   txtH  = MemHandleNew(StrLen(str) + 1);
   if (!txtH)
      return NULL;

   // copy the string to the locked handle.
   StrCopy(MemHandleLock(txtH), str);

   // unlock the string handle.
   MemHandleUnlock(txtH);
   
   // set the field to the handle
   return SetFieldTextFromHandle(fieldID, txtH);
} 

//clears the text from a field:
void  ClearFieldText(Int16 fieldID)
{
   SetFieldTextFromHandle(fieldID, NULL);
}
