/**	
	$Author: dizzy $
	$Revision: 1.6 $
	Last updated: $Date: 2000/09/04 14:23:07 $
	
	$Log: NewAccountForm.c,v $
	Revision 1.6  2000/09/04 14:23:07  dizzy
	General code cleanup

	Copyright (C) 2000 Vlad Mereuta <V.C.Mereuta@warwick.ac.uk>

	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 2 of the License, or
	any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
	
*/

#include "NewTransactionForm.h"

// module globals - placed here to avoid naming conflicy with Accdetails form

static FormPtr	frm;
static Int16	day,month,year;
static Char		date_txt[11];	

static Char*	list_choices[100];
static UInt16	choices_no;
static Boolean	is_debit;
static UInt32	cur_acc;

Boolean NewTrans_Save()
{	
 	transactionStructure 	t;
	UInt32				peer_acc;

	//if the balance is invalid it will be 0
	t.ammount=String2Pence((Char*) GetTextPtr(NEW_TRANS_AMMOUNT));
	if(t.ammount==0) return false; //ammount has to be != 0
	t.payee=(Char*) GetTextPtr(NEW_TRANS_PAYEE);
	if(!t.payee) t.payee="n/a";
	
	t.date.day=day;
	t.date.year=year-1904;
	t.date.month=month;
	t.cleared=FrmGetControlValue(frm,FrmGetObjectIndex(frm,TRANS_CLEARED_CBX));
	
	peer_acc=AccountsDB_FindAccountByName(LstGetSelectionText(GetObjectPtr(NEW_TRANS_ACC_LIST),\
				LstGetSelection(GetObjectPtr(NEW_TRANS_ACC_LIST))),dmAllCategories);
	
	t.method=TransactionsDB_GetTypeIDByName(LstGetSelectionText(GetObjectPtr(NEW_TRANS_TYPE_LIST),\
			LstGetSelection(GetObjectPtr(NEW_TRANS_ACC_LIST))));
	
	if(is_debit) {
		t.source_acc=cur_acc;
		t.dest_acc=peer_acc;
	}
	else {
		t.source_acc=peer_acc;
		t.dest_acc=cur_acc;		
	}
	
	TransactionsDB_NewRecord(&t);

	return true;
}

void NewTrans_InitAccList()
{
	UInt16					recordNo,i;
	MemHandle				rech;
	packedAccountStructure*	pas;
	accountStructure 		as;
	Char					cn[dmCategoryLength];
	UInt32					uid;
		
	choices_no=0;
  	for(i=0,recordNo=0;;i++,recordNo++) {
		//fetch record

		rech=DmQueryNextInCategory(AccountsDB_ref,&recordNo,dmAllCategories);		
		if(!rech) break;

		pas=(packedAccountStructure*) MemHandleLock(rech);
		ErrFatalDisplayIf(!pas, "pas is 0 - aborting");
		if(!pas) return;
		AccountsDB_UnpackAccount(&as,pas);

		//get the uid
		DmRecordInfo(AccountsDB_ref,recordNo,NULL,&uid,NULL);
		if(uid==cur_acc) continue; //do not show the current account as destination
	
		CategoryGetName(AccountsDB_ref,db_GetRecordCategory(AccountsDB_ref,recordNo),cn);
		if (is_debit) { //it is a payment
			if(StrCaselessCompare(cn,"Income")) //accept everything except the accounts in income
				list_choices[choices_no++]=as.acname;					
		}
		else 
			if(StrCaselessCompare(cn,"Expense"))
				list_choices[choices_no++]=as.acname;					

		MemHandleUnlock(rech);
	}
	LstSetListChoices(GetObjectPtr(NEW_TRANS_ACC_LIST),list_choices,choices_no);
}

void NewTrans_InitMethods()
{
	LstSetListChoices(GetObjectPtr(NEW_TRANS_TYPE_LIST),TransactionsDB_types,TransactionsDB_types_no);
}
void NewTrans_UpdateDateField()
{
	DateTemplateToAscii("^0z/^3z/^4r",month,day,year,date_txt,11);
	SetFieldTextFromStr(NEW_TRANS_DATE,date_txt);
}
 
void NewTrans_InitDate()
{
	DateType date;

 	DateSecondsToDate(TimGetSeconds(),& date);
	day=date.day;
	month=date.month;
	year=date.year+1904;
	NewTrans_UpdateDateField();	
}

 
void NewTrans_ChooseDate()
{
	Boolean c;

	c=SelectDay(selectDayByDay,&month,&day,&year,"Choose transaction date");
	if(c) NewTrans_UpdateDateField();
}

Boolean NewTrans_HandleEvent(EventPtr event)
{
	Boolean 	handled = true;
	ControlPtr	ctl;

	// Get the form pointer
	frm = FrmGetActiveForm();

	ctl = GetObjectPtr(NEW_TRANS_ACC_TRIGGER);
	//get the uid of the current account
	DmRecordInfo(AccountsDB_ref,AccountsDB_current_record,NULL,&cur_acc,NULL);

	// Key Events
	switch(event->eType) {

	// Form Open
	case frmOpenEvent:		
		NewTrans_InitDate();

		FrmSetControlGroupSelection(frm,1,NEW_TRANS_PAYMENT);
		is_debit=true;
		
		NewTrans_InitAccList();
		CategorySetTriggerLabel(ctl, "E:Misc");

		NewTrans_InitMethods();
		ctl = GetObjectPtr(NEW_TRANS_TYPE_TRIGGER);
		CategorySetTriggerLabel(ctl, TransactionsDB_types[0]);

		//draw form
		FrmDrawForm(frm);
		break;
	
	// Controls
	case ctlSelectEvent:
		switch( event->data.ctlSelect.controlID ) {
			case NEW_TRANS_OK:
				handled = true;
				//if something went wrong do nothing
				if(! NewTrans_Save()) break;
				//update the main form
				FrmUpdateForm(TRANSACTIONS_FORM,0);				
				FrmReturnToForm(TRANSACTIONS_FORM);
				break;
			case NEW_TRANS_DEPOSIT: //the radio buttons
				handled=true;
				if(!is_debit) break;
				is_debit=false;
				CategorySetTriggerLabel(ctl, "I:Misc");		
				NewTrans_InitAccList();
				break;
			case NEW_TRANS_PAYMENT:
				handled=true;
				if(is_debit) break;
				is_debit=true;
				CategorySetTriggerLabel(ctl, "E:Misc");
				NewTrans_InitAccList();
				break;				
			case NEW_TRANS_CHOOSE_DATE:
				NewTrans_ChooseDate();
				break;
			case NEW_TRANS_CANCEL:
				//cancel button was pressed;
				//simply return to the main form
				handled = true;
				FrmUpdateForm(TRANSACTIONS_FORM,0);
				FrmReturnToForm(TRANSACTIONS_FORM);
				break;
				
			default:
				handled = false;
		};
		break;	
	// Everything else...
	default:
		handled = false;
	}	
	return handled;
}
