/**	
	$Author: dizzy $
	$Revision: 1.6 $
	Last updated: $Date: 2000/09/04 14:23:07 $
	
	Copyright (C) 2000 Vlad Mereuta <V.C.Mereuta@warwick.ac.uk>

	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 2 of the License, or
	any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

	$Log: $
*/

#include "TransactionDetailsForm.h"

//current form
static FormPtr	frm;
//stores the choosen date
static Int16	day,month,year;
//the ascii text coresponding to the date
static Char		date_txt[11];	

//populated with account names to be displayed in the popup
static Char*	list_choices[100];
//no of accounts
static UInt16	choices_no;
//true if the current transaction is a paymen
static Boolean	is_debit;
//uid of the current account
static UInt32	cur_acc;

static transactionStructure t;
static MemHandle			rech;
static Char					p_acc[30];

static Int32	old_ammount,old_src,old_dst;


Boolean TransDetails_Save()
{	
	UInt32	
							peer_acc;
	accountStructure		a;

	//if the balance is invalid it will be 0
	t.ammount=String2Pence((Char*) GetTextPtr(TRANS_DETAILS_AMMOUNT));
	if(t.ammount==0) return false; //ammount has to be != 0
	t.payee=(Char*) GetTextPtr(TRANS_DETAILS_PAYEE);
	if(!t.payee) t.payee="n/a";
	
	t.date.day=day;
	t.date.year=year-1904;
	t.date.month=month;
	t.cleared=FrmGetControlValue(frm,FrmGetObjectIndex(frm,TRANS_CLEARED_CBX));
	
	peer_acc=AccountsDB_FindAccountByName(LstGetSelectionText(GetObjectPtr(TRANS_DETAILS_ACC_LIST),\
				LstGetSelection(GetObjectPtr(TRANS_DETAILS_ACC_LIST))),dmAllCategories);
	
	t.method=TransactionsDB_GetTypeIDByName(LstGetSelectionText(GetObjectPtr(TRANS_DETAILS_TYPE_LIST),\
			LstGetSelection(GetObjectPtr(TRANS_DETAILS_TYPE_LIST))));
	ErrFatalDisplay(TransactionsDB_types[t.method]);
	if(is_debit) {
		t.source_acc=cur_acc;
		t.dest_acc=peer_acc;
	}
	else {
		t.source_acc=peer_acc;
		t.dest_acc=cur_acc;		
	}
	TransactionsDB_PackTransaction(&t,rech);

	//cancel the old transfer
	rech=AccountsDB_GetUnpackedRecord(old_src,&a);

	a.balance+=old_ammount;
	a.eom_balance+=old_ammount;

	//write the data
	AccountsDB_PackAccount(&a,rech);
	rech=AccountsDB_GetUnpackedRecord(old_dst,&a);

	a.balance-=old_ammount;
	a.eom_balance-=old_ammount;

	//write the data
	AccountsDB_PackAccount(&a,rech);

	
	//update the balances for the accounts involved in the transfer
	rech=AccountsDB_GetUnpackedRecord(t.source_acc,&a);

	a.balance-=t.ammount;
	a.eom_balance-=t.ammount;

	//write the data
	AccountsDB_PackAccount(&a,rech);

	rech=AccountsDB_GetUnpackedRecord(t.dest_acc,&a);

	a.balance+=t.ammount;
	a.eom_balance+=t.ammount;

	//write the data
	AccountsDB_PackAccount(&a,rech);

	AccountsDB_totals.dirty=true;
	
	return true;
}

void TransDetails_InitAccList(Boolean use_acc_name)
{
	UInt16					recordNo,i,select=0;
	MemHandle				rh;
	packedAccountStructure*	pas;
	accountStructure 		as;
	Char					cn[dmCategoryLength];
	UInt32					uid;
	ControlPtr				ctl = GetObjectPtr(TRANS_DETAILS_ACC_TRIGGER);
	

		
	choices_no=0;
  	for(i=0,recordNo=0;;i++,recordNo++) {
		//fetch record

		rh=DmQueryNextInCategory(AccountsDB_ref,&recordNo,dmAllCategories);		
		if(!rh) break;

		pas=(packedAccountStructure*) MemHandleLock(rh);
		ErrFatalDisplayIf(!pas, "pas is 0 - aborting");
		if(!pas) return;
		AccountsDB_UnpackAccount(&as,pas);

		//get the uid
		DmRecordInfo(AccountsDB_ref,recordNo,NULL,&uid,NULL);
		if(uid==cur_acc) {
			MemHandleUnlock(rh);
			continue; //do not show the current account as destination
		}
	
		CategoryGetName(AccountsDB_ref,db_GetRecordCategory(AccountsDB_ref,recordNo),cn);
		if(use_acc_name) {
			if(!StrCompare(p_acc,as.acname)) select=choices_no;
		}
		else 
			if(t.source_acc==uid || t.dest_acc==uid) {
				select=choices_no-1;
				StrCopy(p_acc,as.acname);
			}

		
				
		if (is_debit) { //it is a payment
			if(StrCaselessCompare(cn,"Income")) {//accept everything except the accounts in income
				list_choices[choices_no++]=as.acname;					
			}
		}
		else 
			if(StrCaselessCompare(cn,"Expense")) {
				list_choices[choices_no++]=as.acname;					
			}
		MemHandleUnlock(rh);
	}
	LstSetListChoices(GetObjectPtr(TRANS_DETAILS_ACC_LIST),list_choices,choices_no);
	LstSetSelection(GetObjectPtr(TRANS_DETAILS_ACC_LIST),select);
	CategorySetTriggerLabel(ctl, p_acc);	
}

void TransDetails_SetDefaults()
{
	ControlPtr	ctl;
	Char*		s;

	LstSetSelection(GetObjectPtr(TRANS_DETAILS_TYPE_LIST),t.method);

	if(t.source_acc==cur_acc) {
		FrmSetControlGroupSelection(frm,1,TRANS_DETAILS_PAYMENT);
		is_debit=true;
	}
	else {
		FrmSetControlGroupSelection(frm,1,TRANS_DETAILS_DEPOSIT);
		is_debit=false;		
	}
	ctl = GetObjectPtr(TRANS_DETAILS_TYPE_TRIGGER);
	CategorySetTriggerLabel(ctl, TransactionsDB_types[t.method]);

	SetFieldTextFromStr(TRANS_DETAILS_PAYEE,t.payee);
	s=Pence2String(t.ammount);
	SetFieldTextFromStr(TRANS_DETAILS_AMMOUNT,s);

	//free memory allocated by Pence2String
	MemPtrFree(s);
	
}

void TransDetails_InitMethods()
{
	LstSetListChoices(GetObjectPtr(TRANS_DETAILS_TYPE_LIST),TransactionsDB_types,TransactionsDB_types_no);
}
void TransDetails_UpdateDateField()
{
	DateTemplateToAscii("^0z/^3z/^4r",month,day,year,date_txt,11);
	SetFieldTextFromStr(TRANS_DETAILS_DATE,date_txt);
	
}
 
void TransDetails_InitDate()
{
	day=t.date.day;
	month=t.date.month;
	year=t.date.year+1904;
	TransDetails_UpdateDateField();	
}

 
void TransDetails_ChooseDate()
{
	Boolean c;

	c=SelectDay(selectDayByDay,&month,&day,&year,"Choose transaction date");
	if(c) TransDetails_UpdateDateField();
}

Boolean TransDetails_HandleEvent(EventPtr event)
{
	Boolean 	handled = true;
	packedTransactionStructure* pts;

	// Get the form pointer
	frm = FrmGetActiveForm();

	//get the uid of the current account
	DmRecordInfo(AccountsDB_ref,AccountsDB_current_record,NULL,&cur_acc,NULL);

		
	// Key Events
	switch(event->eType) {

	// Form Open
	case frmOpenEvent:		
		rech=DmQueryRecord(TransactionsDB_ref,TransactionsDB_current_record);
		pts=(packedTransactionStructure*) MemHandleLock(rech);
		TransactionsDB_UnpackTransaction(&t,pts);
		MemHandleUnlock(rech);

		old_ammount=t.ammount;
		old_src=t.source_acc;
		old_dst=t.dest_acc;
		TransDetails_InitDate();
		
		TransDetails_InitAccList(false);
		TransDetails_InitMethods();
		TransDetails_SetDefaults();
		//draw form
		FrmDrawForm(frm);
		break;
	
	// Controls
	case ctlSelectEvent:
		switch( event->data.ctlSelect.controlID ) {
			case TRANS_DETAILS_OK:
				handled = true;
				//if something went wrong do nothing
				if(! TransDetails_Save()) break;
				//update the main form
				FrmUpdateForm(TRANSACTIONS_FORM,0);				
				FrmReturnToForm(TRANSACTIONS_FORM);
				break;
			case TRANS_DETAILS_DEPOSIT: //the radio buttons
				handled=true;
				if(!is_debit) break;
				is_debit=false;
				StrCopy(p_acc, "I:Misc");		
				TransDetails_InitAccList(true);
				break;
			case TRANS_DETAILS_PAYMENT:
				handled=true;
				if(is_debit) break;
				is_debit=true;
				StrCopy(p_acc, "E:Misc");		
				TransDetails_InitAccList(true);
				break;				
			case TRANS_DETAILS_CHOOSE_DATE:
				TransDetails_ChooseDate();
				break;
			case TRANS_DETAILS_CANCEL:
				//cancel button was pressed;
				//simply return to the main form
				handled = true;
				FrmUpdateForm(TRANSACTIONS_FORM,0);
				FrmReturnToForm(TRANSACTIONS_FORM);
				break;
				
			default:
				handled = false;
		};
		break;	
	// Everything else...
	default:
		handled = false;
	}	
	return handled;
}
