/**	
	$Author: dizzy $
	$Revision: 1.2 $
	Last updated: $Date: 2000/09/13 11:50:27 $
	
	Copyright (C) 2000 Vlad Mereuta <V.C.Mereuta@warwick.ac.uk>

	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 2 of the License, or
	any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

	$Log: TransactionsDB.c,v $
	Revision 1.2  2000/09/13 11:50:27  dizzy
	The transaction module is fully functional
*/

//define this to ignore the external definition of the vars below in the header file
#define __TRANSACTIONS_DB_C__

#include "TransactionsDB.h"


//the variables below have to be accessed in different object files

UInt16			TransactionsDB_types_no=6;
DmOpenRef		TransactionsDB_ref;
UInt32			TransactionsDB_num_cat;
UInt16			TransactionsDB_current_cat=dmAllCategories;
Int16			TransactionsDB_current_record;
Char*			TransactionsDB_types[6]={"Card","Cash","Cheque","Standing Order","Direct Debit","Transfer"};

payment_method	TransactionsDB_GetTypeIDByName(Char* name)
{
	if(!StrCompare(name,"Card")) return pm_card;
	if(!StrCompare(name,"Cash")) return pm_cash;
	if(!StrCompare(name,"Cheque")) return pm_cheque;
	if(!StrCompare(name,"Standing Order")) return pm_so;
	if(!StrCompare(name,"Direct Debit")) return pm_dd;
	if(!StrCompare(name,"Transfer")) return pm_transfer;
	//top stop the compiler from complaining ;)
	ErrFatalDisplay("Name not correct");
	return 0;
}

Int32	TransactionsDB_RecordGetAmmount(UInt32 id)
{
	Int16						record;
	MemHandle					rech;
	packedTransactionStructure* pts;
	Int32						ret_val;

	DmFindRecordByID(TransactionsDB_ref,id,&record);

	rech=DmQueryRecord(TransactionsDB_ref,record);
	ErrFatalDisplayIf(!rech, "rech is 0 - aborting");

	pts=(packedTransactionStructure*) MemHandleLock(rech);
	ErrFatalDisplayIf(!pts, "pts is 0 - aborting");

	ret_val=pts->ammount;
	MemHandleUnlock(rech);
	return ret_val;
}

void	TransactionsDB_RecordGetDate(UInt32 id, Char* s, const Char* format,UInt16 len)
{
	Int16						record;
	MemHandle					rech;
	packedTransactionStructure* pts;

	DmFindRecordByID(TransactionsDB_ref,id,&record);

	rech=DmQueryRecord(TransactionsDB_ref,record);
	ErrFatalDisplayIf(!rech, "rech is 0 - aborting");

	pts=(packedTransactionStructure*) MemHandleLock(rech);
	ErrFatalDisplayIf(!pts, "pts is 0 - aborting");
	
	DateTemplateToAscii(format,pts->date.month,pts->date.day,pts->date.year,s,len);

	MemHandleUnlock(rech);
}


void	TransactionsDB_GetUnpackedRecord(UInt32 id,transactionStructure* ts)
{
	Int16						record;
	MemHandle					rech;
	packedTransactionStructure*	pts;

	DmFindRecordByID(TransactionsDB_ref,id,&record);
	
	rech=DmQueryRecord(TransactionsDB_ref,record);
	if(!rech) return;
	ErrFatalDisplayIf(!rech, "rech is 0 - aborting");

	pts=(packedTransactionStructure *) MemHandleLock(rech);
	ErrFatalDisplayIf(!pts, "pts is 0 - aborting");
	if(!pts) return;

	TransactionsDB_UnpackTransaction(ts,pts);
	MemHandleUnlock(rech);
}


void	TransactionsDB_RecordGetDestAcc(UInt32 id,accountStructure* as)
{
	Int16						record;
	MemHandle					rech;
	packedTransactionStructure*	pts;


	DmFindRecordByID(TransactionsDB_ref,id,&record);

	rech=DmQueryRecord(TransactionsDB_ref,record);
	ErrFatalDisplayIf(!rech, "rech is 0 - aborting");

	pts=(packedTransactionStructure*) MemHandleLock(rech);
	ErrFatalDisplayIf(!pts, "pts is 0 - aborting");

	AccountsDB_GetUnpackedRecord(pts->dest_acc,as);
	MemHandleUnlock(rech);
	
}

void	TransactionsDB_RecordGetSrcAcc(UInt32 id,accountStructure* as)
{
	Int16						record;
	MemHandle					rech;
	packedTransactionStructure*	pts;


	DmFindRecordByID(TransactionsDB_ref,id,&record);

	rech=DmQueryRecord(TransactionsDB_ref,record);
	ErrFatalDisplayIf(!rech, "rech is 0 - aborting");

	pts=(packedTransactionStructure*) MemHandleLock(rech);
	ErrFatalDisplayIf(!pts, "pts is 0 - aborting");

	AccountsDB_GetUnpackedRecord(pts->source_acc,as);
	MemHandleUnlock(rech);
	
}

void TransactionsDB_SetRecordCategory(UInt16 index, UInt16 cat)
{
	db_SetRecordCategory(TransactionsDB_ref,index,cat);
}

void TransactionsDB_UnpackTransaction (transactionStructure* trans, const packedTransactionStructure* packed_trans)
{
	Char*	s = packed_trans->payee;
	
	trans->source_acc = packed_trans->source_acc;
	trans->dest_acc = packed_trans->dest_acc;
	trans->date = packed_trans->date;
	trans->cleared = packed_trans->cleared;
	trans->ammount = packed_trans->ammount;
	trans->method = packed_trans->method;

	trans->payee = s;
	s += StrLen(s) + 1;
	trans->note = s;
}

void TransactionsDB_PackTransaction (transactionStructure* trans, MemHandle db_entry)
{
	// figure out necessary size
	UInt32	length = 0;
	MemPtr	s;
	UInt32	offset = 0;

	//for some reason DateType seems to have the size 1 byte bigger then what sizeof reports
	length = sizeof(trans->source_acc) + sizeof(trans->dest_acc) + sizeof(trans->date) + sizeof(trans->cleared)+ \
		 sizeof(trans->ammount) + sizeof(trans->method) +StrLen(trans->payee) + StrLen(trans->note) + 3;  // 4 for 2 string terminators

	// resize the VoidHand
	if (MemHandleResize(db_entry, length) == 0) {
		// copy the fields
		s = MemHandleLock(db_entry);
		offset = 0;
		DmWrite(s, offset, (MemPtr) &trans->source_acc,sizeof(trans->source_acc));
		offset += sizeof(trans->source_acc);
		DmWrite(s, offset, (MemPtr) &trans->dest_acc,sizeof(trans->dest_acc));
		offset += sizeof(trans->dest_acc);
		DmWrite(s, offset, (MemPtr) &trans->date,sizeof(DateType));
		offset += sizeof(DateType)+1;
		DmWrite(s, offset, (MemPtr) &trans->cleared,sizeof(trans->cleared));
		offset += sizeof(trans->cleared);
		DmWrite(s, offset, (MemPtr) &trans->ammount,sizeof(trans->ammount));
		offset += sizeof(trans->ammount);
		DmWrite(s, offset, (MemPtr) &trans->method,sizeof(trans->method));
		offset += sizeof(trans->method);
		
		DmStrCopy(s, offset, (Char*) trans->payee);
		offset += StrLen(trans->payee) + 1;
		DmStrCopy(s, offset, (Char*) trans->note);
				
		MemHandleUnlock(db_entry);
	}
}

Err		TransactionsDB_Init()
{
	UInt32	mode = dmModeReadWrite;
	Err	err = 0;
	Boolean	created;


	// Find the database.  If it doesn't exist, create it.
	err=db_OpenOrCreate(&TransactionsDB_ref, TRANSACTIONSDB_TYPE,PALM_COINS_CREATOR, mode,0, TRANSACTIONSDB_NAME, &created);

	return err;
}

void	TransactionsDB_Close()
{
	DmCloseDatabase(TransactionsDB_ref);
}


UInt16	TransactionsDB_NewRecord(transactionStructure* s)
{
	MemHandle					h;
	UInt16						index=0;
	accountStructure		 	a;
	MemHandle					rech;
	
	h=DmNewRecord(TransactionsDB_ref, &index,1);
	if (h) 	TransactionsDB_PackTransaction(s,h);
		else 	ErrFatalDisplay("No space left?");
	DmReleaseRecord(TransactionsDB_ref, index, true); //true sets the dirty bit			


	//update the balances for the accounts involved in the transfer
	rech=AccountsDB_GetUnpackedRecord(s->source_acc,&a);

	a.balance-=s->ammount;
	a.eom_balance-=s->ammount;

	//write the data
	AccountsDB_PackAccount(&a,rech);

	rech=AccountsDB_GetUnpackedRecord(s->dest_acc,&a);

	a.balance+=s->ammount;
	a.eom_balance+=s->ammount;

	//write the data
	AccountsDB_PackAccount(&a,rech);

	AccountsDB_totals.dirty=true;
	
	return index;
}
