/**	
	Code to access the database containing the Transaction information

	$Author: dizzy $
	$Revision: 1.2 $
	Last updated: $Date: 2000/09/13 11:50:27 $
	
	$Log: TransactionsDB.h,v $
	Revision 1.2  2000/09/13 11:50:27  dizzy
	The transaction module is fully functional

	Copyright (C) 2000 Vlad Mereuta <V.C.Mereuta@warwick.ac.uk>

	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 2 of the License, or
	any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
	
*/

#ifndef __TRANSACTIONS_DB__
#define __TRANSACTIONS_DB__

#include <PalmOS.h>
#include <DateTime.h>
#include <FeatureMgr.h>

#include "PalmCoinsRsc.h"
#include "MiscFunc.h"
#include "AccountsDB.h"

/// type of the Transaction database
#define TRANSACTIONSDB_TYPE		'DATA'
/// db name
#define TRANSACTIONSDB_NAME	"PalmCoins-Transactions-DyCo"

//define this variables as extern if the file is not included from TransactionsDB.c
#ifndef __TRANSACTIONS_DB_C__
	/// reference to the database
	extern DmOpenRef		TransactionsDB_ref;
	/// total number of categories
	extern UInt32			TransactionsDB_num_cat;
	/// the currenct category
	extern UInt16			TransactionsDB_current_cat;
	/// the current record (to be displayed/edited/etc)
	extern Int16			TransactionsDB_current_record;
	///the strings corresponding to the payment method enum
	extern Char*			TransactionsDB_types[6];
	///the number of types stored in TransactionDB_types
	extern UInt16			TransactionsDB_types_no;
#endif

/// payment method
typedef enum { pm_card, pm_cash, pm_cheque, pm_so, pm_dd, pm_transfer } payment_method;
	
	
///the packed structure of a transaction - see the unpacked struct for explanations
typedef struct {
	UInt32			source_acc;
	UInt32			dest_acc;
	DateType		date;
	Boolean			cleared;
	Int32			ammount;
	payment_method	method;
	Char			payee[1];
} packedTransactionStructure;

///unpacked structure of an Transaction
typedef struct {
	/// the Transaction where the transaction originates
	UInt32			source_acc;
	/// destination Transaction for the transaction
	UInt32			dest_acc;
	///date on which the transaction was made
	DateType		date;
	///wether the transaction is clearead or not
	Boolean			cleared;
	/// transaction total
	Int32			ammount;
	///tthe method of payment
	payment_method	method;
	/// person/company 
	Char*			payee;
	/// note on the transaction	
	Char*			note;
} transactionStructure;



// ***** FUNCTIONS

/** 
	sets the category of a given record
	@param index	the index of the record
	@param cat		the record category
*/
void	TransactionsDB_SetRecordCategory(UInt16 index, UInt16 cat);


/// unpacks an Transaction record
void	TransactionsDB_UnpackTransaction (transactionStructure* trans, const packedTransactionStructure* packed_trans);

/// packs an Transaction record
void	TransactionsDB_PackTransaction (transactionStructure* trans, MemHandle db_entry);

/// fetch an unpacked record by passing its unique id
void	TransactionsDB_GetUnpackedRecord(UInt32 id,transactionStructure* ts);


/** 
	get the date of the transaction
	@param id unique id of the record
	@param s pointer to allocated string, where the formated date will be writen to
	@param format format to write the date in
	@param len maximum length of s
	@see DateTemplateToAscii
*/
void	TransactionsDB_RecordGetDate(UInt32 id, Char* s, const Char* format,UInt16 len);

/** 
	@param id unique id of the record
	@return the ammount involved in a transaction		
*/
Int32	TransactionsDB_RecordGetAmmount(UInt32 id);

/** 
	fetch the destination account informations
	@param id unique id of the record
	@param as pointer to an accountStructure where the information will be saved
 */			
void	TransactionsDB_RecordGetDestAcc(UInt32 id,accountStructure* as);

/** 
	fetch the destination account informations
	@param id unique id of the record
	@param as pointer to an accountStructure where the information will be saved
 */			
void	TransactionsDB_RecordGetSrcAcc(UInt32 id,accountStructure* as);

/**
	Returns the id of a given transaction type string
	@param name the name for the transaction_type
	@return the associated id
*/
payment_method		TransactionsDB_GetTypeIDByName(Char* name);

/// initialize the databases
 Err	TransactionsDB_Init();

/// closes the database
void 	TransactionsDB_Close();

///init the Transactions DB with some default records
void	TransactionsDB_InitializeTransactions();

/**
	creates a new record and update the balances of the two involved accounts
*/
UInt16	TransactionsDB_NewRecord(transactionStructure* s);


#endif
