/**	
	$Author: dizzy $
	$Revision: 1.1 $
	Last updated: $Date: 2000/09/13 11:50:27 $
	
	$Log: TransactionsTable.c,v $
	Revision 1.1  2000/09/13 11:50:27  dizzy
	The transaction module is fully functional

	Copyright (C) 2000 Vlad Mereuta <V.C.Mereuta@warwick.ac.uk>

	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 2 of the License, or
	any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
	
*/

#define __TRANSACTIONS_TABLE_C__

#include "TransactionsTable.h"

// the uniqueid for top visible items
UInt16		TransactionsTable_top_visible_item=0;
// the max no of rows that can be displayed
UInt16		TransactionsTable_rows;
// no of elements that could be loaded in the table
UInt16		TransactionsTable_total_items;

//name all the columns
UInt16		TransactionsTable_date_col=0;
UInt16		TransactionsTable_dest_col=1;
UInt16		TransactionsTable_ammount_col=2;

// the table
static TablePtr		table;
static UInt32		cur_acc; //uniqueid of the current record in accounts db


void TransactionsTable_ShowScrollArrows()
{
    ControlPtr	ptrup, ptrdown;
    Boolean		enable;
	char 		*pageupchr="\001", *pagedownchr="\002";
	char 		*pageupgreychr="\003", *pagedowngreychr="\004";

    ptrup = GetObjectPtr(TRANS_PGUP);
    ptrdown = GetObjectPtr(TRANS_PGDN);

    if ((TransactionsTable_total_items > TransactionsTable_rows) || TransactionsTable_top_visible_item) {
    	// first element is not 0, hence we can scroll up
        if (TransactionsTable_top_visible_item) {
            CtlSetLabel(ptrup, pageupchr);
            enable = true;
        }
        else {
            CtlSetLabel(ptrup, pageupgreychr);
            enable = false;
        }
        
		CtlShowControl(ptrup);
        CtlSetEnabled(ptrup, enable);
	
		// enable scrolling down if there are enough elements
        if ((TransactionsTable_top_visible_item+TransactionsTable_rows) < TransactionsTable_total_items) {
            CtlSetLabel(ptrdown, pagedownchr);
            enable = true;
        }
        else {
            CtlSetLabel(ptrdown, pagedowngreychr);
            enable = false;
        }
        CtlShowControl(ptrdown);
        CtlSetEnabled(ptrdown, enable);
    }
    else {
        CtlHideControl(ptrup);
        CtlHideControl(ptrdown);
		CtlSetEnabled(ptrdown,false);
		CtlSetEnabled(ptrup,false);		
    }
}

void TransactionsTable_DrawMoney(void* table, Int16 row, Int16 column, RectangleType* bounds)
{
	Char					*s;
	transactionStructure 	ts;

	TransactionsDB_GetUnpackedRecord(TblGetRowData(table,row),&ts);
	if(ts.dest_acc != cur_acc)  //debit show a minus, otherwise 
		s=Pence2String(-TransactionsDB_RecordGetAmmount(TblGetRowData(table,row)));
	else s=Pence2String(TransactionsDB_RecordGetAmmount(TblGetRowData(table,row)));

	WinDrawChars(s,StrLen(s),bounds->topLeft.x,bounds->topLeft.y);

	//free memory allocated by Pence2String
	MemPtrFree(s);
}

void TransactionsTable_DrawDate(void* table, Int16 row, Int16 column, RectangleType* bounds)
{
	Char		s[6];

	TransactionsDB_RecordGetDate(TblGetRowData(table,row),s,"^0z/^3z",6);

	WinDrawChars(s,StrLen(s),bounds->topLeft.x,bounds->topLeft.y);
}


void TransactionsTable_DrawAcc(void* table, Int16 row, Int16 column, RectangleType* bounds)
{
	accountStructure		as;
	transactionStructure 	ts;

	TransactionsDB_GetUnpackedRecord(TblGetRowData(table,row),&ts);
	
	if(coins_pref.disp_payee) // display the payee
		WinDrawChars(ts.payee,StrLen(ts.payee),bounds->topLeft.x,bounds->topLeft.y);
	else { //display the peer account
		if(ts.dest_acc != cur_acc) //payment
			TransactionsDB_RecordGetDestAcc(TblGetRowData(table,row),&as);
		else 
			TransactionsDB_RecordGetSrcAcc(TblGetRowData(table,row),&as);

		WinDrawChars(as.acname,StrLen(as.acname),bounds->topLeft.x,bounds->topLeft.y);
	}
}

void TransactionsTable_LoadData()
{
	UInt16					recordNo=0,row;
	UInt32					uniqueID;
	transactionStructure	ts;
	int						read,t;
	
	TblUnhighlightSelection(table);
	
	t=DmNumRecordsInCategory(TransactionsDB_ref,TransactionsDB_current_cat);
//	TransactionsTable_total_items=DmNumRecordsInCategory(TransactionsDB_ref,TransactionsDB_current_cat);
	for(TransactionsTable_total_items=0,read=0;read<t;read++) {
		if(!DmQueryNextInCategory(TransactionsDB_ref,&recordNo,TransactionsDB_current_cat)) break;				
//			ErrFatalDisplay("here one");		
		DmRecordInfo(TransactionsDB_ref,recordNo,NULL,&uniqueID,NULL);
		TransactionsDB_GetUnpackedRecord(uniqueID,&ts);
		recordNo++; 
		if((ts.source_acc == cur_acc) || (ts.dest_acc == cur_acc)) { //transaction belongs in this table
			TransactionsTable_total_items++;
		}
		
	}

	
	recordNo=TransactionsTable_top_visible_item;
	//show the scrolling arrows if necessary
	TransactionsTable_ShowScrollArrows(TransactionsTable_total_items);
	
  	for(row=0,read=0;row<TransactionsTable_rows;row++,read++) {
		//if we run out of items to display just set the rows to unusable
		if(row+TransactionsTable_top_visible_item>=t /*TransactionsTable_total_items*/ || read>=t/*TransactionsTable_total_items*/) {
			TblSetRowUsable(table,row,false);
			continue;
		} else TblSetRowUsable(table,row,true);
		
		//fetch next record
		if(!DmQueryNextInCategory(TransactionsDB_ref,&recordNo,TransactionsDB_current_cat)) break;				
		
		DmRecordInfo(TransactionsDB_ref,recordNo,NULL,&uniqueID,NULL);
		TransactionsDB_GetUnpackedRecord(uniqueID,&ts);
		if((ts.source_acc != cur_acc) && (ts.dest_acc != cur_acc)) { //transaction does not belong in this table
			row--;
			recordNo++; 
			
			continue;
		}
		
		TblSetRowData(table,row,uniqueID);
		
		//start searching from the next loop the following time
		recordNo++; 

	}
}

void TransactionsTable_Unhilight()
{
	TblUnhighlightSelection(table);

}

void TransactionsTable_Init()
{
	UInt16		row;
	
	//init globals
	table=(TablePtr) GetObjectPtr(TRANS_TABLE);
	
	//set the uid for the account described by this table
	DmRecordInfo(AccountsDB_ref,AccountsDB_current_record,NULL,&cur_acc,NULL);
	TransactionsTable_Unhilight();

	//current_cat is set to dmAllRecs so that we get all the records except the delted ones	
//	TransactionsTable_total_items=DmNumRecordsInCategory(TransactionsDB_ref,TransactionsDB_current_cat);
	


	ErrFatalDisplayIf(!table, "table is null!!!");

	//the numbers are drawn with our custom procedure
	TblSetCustomDrawProcedure(table, TransactionsTable_date_col, TransactionsTable_DrawDate);
	TblSetCustomDrawProcedure(table, TransactionsTable_dest_col, TransactionsTable_DrawAcc);
	TblSetCustomDrawProcedure(table, TransactionsTable_ammount_col, TransactionsTable_DrawMoney);

	//get max number of visible rows
	TransactionsTable_rows = TblGetNumberOfRows(table);

	//set style for all cells
	for (row = 0; row < TransactionsTable_rows; row++) {
		TblSetItemStyle(table, row, TransactionsTable_date_col, customTableItem);
		TblSetItemStyle(table, row, TransactionsTable_dest_col, customTableItem);
		TblSetItemStyle(table, row, TransactionsTable_ammount_col, customTableItem);
	}
	
/** 	//set all columns as usable
	TblSetColumnUsable(table, TransactionsTable_name_col, true);
	if(coins_pref.disp_eom_col) {
		TblSetColumnUsable(table, TransactionsTable_eom_balance_col,true);
		TblSetColumnWidth(table,TransactionsTable_name_col,66);
	}
	else {
		//dont display the column
		TblSetColumnUsable(table, TransactionsTable_eom_balance_col,false);
		//make the anme column bigger
		TblSetColumnWidth(table,TransactionsTable_name_col,111);
	}
 */		
	TblSetColumnUsable(table, TransactionsTable_date_col, true);
 	TblSetColumnUsable(table,TransactionsTable_dest_col, true);
	TblSetColumnUsable(table, TransactionsTable_ammount_col, true);
	
	TransactionsTable_LoadData();
}
